@@ -91,6 +91,13 @@
             unknown-action-name
             unknown-action-service
 
+            &action-runtime-error
+            action-runtime-error?
+            action-runtime-error-service
+            action-runtime-error-action
+            action-runtime-error-key
+            action-runtime-error-arguments
+
             condition->sexp))
 
 ;; Type of service actions.
@@ -115,7 +122,12 @@
      '())))
 
 ;; Respawning CAR times in CDR seconds will disable the service.
-(define respawn-limit (cons 5 5))
+;;
+;; XXX: The terrible hack in (shepherd) using SIGALRM to work around
+;; unreliable SIGCHLD delivery means that it might take up to 1 second for
+;; SIGCHLD to be delivered.  Thus, arrange for the car to be lower than the
+;; cdr.
+(define respawn-limit '(5 . 7))
 
 (define (respawn-limit-hit? respawns times seconds)
   "Return true of RESPAWNS, the list of times at which a given service was
@@ -304,9 +316,6 @@ wire."
 			     problem)
                (call-with-blocked-asyncs
                 (lambda ()
-                  ;; Reset the list of respawns.
-                  (slot-set! obj 'last-respawns '())
-
                   ;; Start the service itself.  Asyncs are blocked so that if
                   ;; the newly-started process dies immediately, the SIGCHLD
                   ;; handler is invoked later, once we have set the 'running'
@@ -330,7 +339,7 @@ wire."
 ;; latter fails, continue anyway.  Return `#f' if it could be stopped.
 (define-method (stop (obj <service>) . args)
   ;; Block asyncs so the SIGCHLD handler doesn't execute concurrently.
-  ;; Notably, that makes sure the handler process the SIGCHLD for OBJ's
+  ;; Notably, that makes sure the handler processes the SIGCHLD for OBJ's
   ;; process once we're done; otherwise, it could end up respawning OBJ.
   (call-with-blocked-asyncs
    (lambda ()
@@ -367,6 +376,9 @@ wire."
                ;; OBJ is no longer running.
                (slot-set! obj 'running #f)
 
+               ;; Reset the list of respawns.
+               (slot-set! obj 'last-respawns '())
+
                ;; Status message.
                (let ((name (canonical-name obj)))
                  (if (running? obj)
@@ -917,8 +929,8 @@ child left."
           ;; because this procedure is supposed to be called only upon
           ;; SIGCHLD.
           (let ((errno (system-error-errno args)))
-            (local-output "error: 'waitpid' unexpectedly failed with: ~s"
-                          (strerror errno))
+            (local-output "warning: 'waitpid' ~a failed unexpectedly: ~a"
+                          what (strerror errno))
             '(0 . #f)))))))
 
 (define (respawn-service signum)
@@ -1109,7 +1121,7 @@ file when persistence is enabled."
     #:docstring "The root service is used to operate on shepherd itself."
     #:provides '(root shepherd)
     #:requires '()
-    #:respawn #f
+    #:respawn? #f
     #:start (lambda args
 	      (when (isatty? (current-output-port))
                 (display-version))
